@@ -3,6 +3,7 @@
 
     SPDX-FileCopyrightText: 2009 Eckhart Wörner <ewoerner@kde.org>
     SPDX-FileCopyrightText: 2010 Frederik Gladhorn <gladhorn@kde.org>
+    SPDX-FileCopyrightText: 2024 Harald Sitter <sitter@kde.or>
 
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 
@@ -10,7 +11,7 @@
 
 #pragma once
 
-#include <attica/platformdependent.h>
+#include <attica/platformdependent_v3.h>
 
 #include <QHash>
 
@@ -18,10 +19,10 @@
 
 namespace Attica
 {
-class KdePlatformDependent : public QObject, public Attica::PlatformDependent
+class KdePlatformDependent : public Attica::PlatformDependentV3
 {
     Q_OBJECT
-    Q_INTERFACES(Attica::PlatformDependent)
+    Q_INTERFACES(Attica::PlatformDependentV3)
     Q_PLUGIN_METADATA(IID "org.kde.attica-kde")
 
 public:
@@ -41,15 +42,20 @@ public:
     bool loadCredentials(const QUrl &baseUrl, QString &user, QString &password) override;
     bool askForCredentials(const QUrl &baseUrl, QString &user, QString &password) override;
     QNetworkAccessManager *nam() override;
+    QNetworkReply *deleteResource(const QNetworkRequest &request) override;
+    QNetworkReply *put(const QNetworkRequest &request, QIODevice *data) override;
+    QNetworkReply *put(const QNetworkRequest &request, const QByteArray &data) override;
+    bool isReady() override;
 
 private:
     QNetworkRequest addOAuthToRequest(const QNetworkRequest &request);
     QNetworkRequest removeAuthFromRequest(const QNetworkRequest &request);
-    QString getAccessToken(const QUrl &baseUrl) const;
+    void loadAccessToken();
 
     KSharedConfigPtr m_config;
     QNetworkAccessManager *m_accessManager{nullptr};
     QHash<QString, QPair<QString, QString>> m_passwords;
+    QString m_accessToken;
 };
 
 }
