@@ -6,8 +6,6 @@
 
 #include "inputdevice.h"
 
-#include <QDBusError>
-#include <QDBusInterface>
 #include <QList>
 
 #include "logging.h"
@@ -63,6 +61,11 @@ InputDevice::InputDevice(const QString &dbusName, QObject *parent)
     connect(this, &InputDevice::outputAreaChanged, this, &InputDevice::needsSaveChanged);
     connect(this, &InputDevice::enabledChanged, this, &InputDevice::needsSaveChanged);
     connect(this, &InputDevice::mapToWorkspaceChanged, this, &InputDevice::needsSaveChanged);
+    connect(this, &InputDevice::pressureCurveChanged, this, &InputDevice::needsSaveChanged);
+    connect(this, &InputDevice::inputAreaChanged, this, &InputDevice::needsSaveChanged);
+    connect(this, &InputDevice::pressureRangeMinChanged, this, &InputDevice::needsSaveChanged);
+    connect(this, &InputDevice::pressureRangeMaxChanged, this, &InputDevice::needsSaveChanged);
+    connect(this, &InputDevice::calibrationMatrixChanged, this, &InputDevice::needsSaveChanged);
 }
 
 void InputDevice::save()
@@ -73,12 +76,17 @@ void InputDevice::save()
     m_outputArea.save();
     m_enabled.save();
     m_mapToWorkspace.save();
+    m_pressureCurve.save();
+    m_inputArea.save();
+    m_pressureRangeMin.save();
+    m_pressureRangeMax.save();
+    m_calibrationMatrix.save();
 }
 
 bool InputDevice::isSaveNeeded() const
 {
     return m_leftHanded.changed() || m_orientation.changed() || m_outputName.changed() || m_outputArea.changed() || m_enabled.changed()
-        || m_mapToWorkspace.changed();
+        || m_mapToWorkspace.changed() || m_pressureCurve.changed() || m_inputArea.changed() || m_pressureRangeMin.changed() || m_pressureRangeMax.changed() || m_calibrationMatrix.changed();
 }
 
 void InputDevice::defaults()
@@ -89,12 +97,17 @@ void InputDevice::defaults()
     m_outputArea.resetFromDefaults();
     m_enabled.resetFromDefaults();
     m_mapToWorkspace.resetFromDefaults();
+    m_pressureCurve.resetFromDefaults();
+    m_inputArea.resetFromDefaults();
+    m_pressureRangeMin.resetFromDefaults();
+    m_pressureRangeMax.resetFromDefaults();
+    m_calibrationMatrix.resetFromDefaults();
 }
 
 bool InputDevice::isDefaults() const
 {
     return m_leftHanded.isDefaults() && m_orientation.isDefaults() && m_outputName.isDefaults() && m_outputArea.isDefaults() && m_enabled.isDefaults()
-        && m_mapToWorkspace.isDefaults();
+        && m_mapToWorkspace.isDefaults() && m_pressureCurve.isDefaults() && m_inputArea.isDefaults() && m_pressureRangeMin.isDefaults() && m_pressureRangeMax.isDefaults() && m_calibrationMatrix.isDefaults();
 }
 
 void InputDevice::load()
@@ -105,6 +118,11 @@ void InputDevice::load()
     m_outputArea.resetFromSaved();
     m_enabled.resetFromSaved();
     m_mapToWorkspace.resetFromSaved();
+    m_pressureCurve.resetFromSaved();
+    m_inputArea.resetFromSaved();
+    m_pressureRangeMin.resetFromSaved();
+    m_pressureRangeMax.resetFromSaved();
+    m_calibrationMatrix.resetFromSaved();
 }
 
 void InputDevice::setOrientation(int ori)
@@ -127,6 +145,11 @@ void InputDevice::setOutputArea(const QRectF &outputArea)
     m_outputArea.set(outputArea);
 }
 
+void InputDevice::setInputArea(const QRectF &inputArea)
+{
+    m_inputArea.set(inputArea);
+}
+
 void InputDevice::setEnabled(bool enabled)
 {
     m_enabled.set(enabled);
@@ -136,3 +159,43 @@ void InputDevice::setMapToWorkspace(bool mapToWorkspace)
 {
     m_mapToWorkspace.set(mapToWorkspace);
 }
+
+void InputDevice::setPressureCurve(const QString &curve)
+{
+    m_pressureCurve.set(curve);
+}
+
+bool InputDevice::pressureCurveIsDefault() const
+{
+    return m_pressureCurve.isDefaults();
+}
+
+QString InputDevice::serializeMatrix(const QMatrix4x4 &matrix)
+{
+    QString result;
+    for (int i = 0; i < 16; i++) {
+        result.append(QString::number(matrix.constData()[i]));
+        if (i != 15) {
+            result.append(QLatin1Char(','));
+        }
+    }
+    return result;
+}
+
+QMatrix4x4 InputDevice::deserializeMatrix(const QString &matrix)
+{
+    const auto items = QStringView(matrix).split(QLatin1Char(','));
+    if (items.size() == 16) {
+        QList<float> data;
+        data.reserve(16);
+        std::ranges::transform(std::as_const(items), std::back_inserter(data), [](const QStringView &item) {
+            return item.toFloat();
+        });
+
+        return QMatrix4x4{data.constData()};
+    }
+
+    return QMatrix4x4{};
+}
+
+#include "moc_inputdevice.cpp"
